#!/usr/bin/env ruby1.8
#==Synopsis
# Waits for current load average to drop below given threshold before 
# continuing. Usefull for batch runs that require certain conditions 
# without the host its being executed.
#
#==Usage
#
# Usage: waitforloadavg [OPTIONS] 
# -h, --help
# -q, --quiet
#    Do not print anything while running
# -d, --decreasing
#    Wait before short interval is lower than mid-range interval 
#    which usually means that load average is dropping
# -l, --load [FLOAT] 
#    Waits before short interval load average is exact or below
#    given value. If given load average is reached, exit code
#    will be 0
# -t, --time [INTEGER]
#    Breaks the wait loop before given condition is met if amount of 
#    seconds [INTEGER] has passed. 
# -f, --fail
#    if parameter -t is give, sets the exitcode to 1 to indicate that
#    timeout really happened
# -p, --poll [FLOAT]
#    Amount of seconds to wait between polling the load average. 
#    Defaults to 0.9
#
#==Warning
# One must use either load or decreasing parameters or combine then 
# both 
# 
#==Author
# Jani Mikkonen <jani@mikkonen.biz>
# My livingroom, 2010, writing code for my amuzement
#
#==Copyright
# waitloadavg - waits until loadavg drops below threshold
# Copyright (C) 2010 Jani Mikkonen 
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require 'getoptlong'
require 'rdoc/usage'


# either initializing the variable or setting the scope here
finished = false
timeout = false
expected = 100.0
loadDecreasing = false
maxTimeToWait = 0
quiet = false
optsSatisfied = false
exitVal = 0
outBuf = ""
looping = true
pollTime = 0.9
failOnTimeout = false

opts = GetoptLong.new(
    ['--help', '-h', GetoptLong::NO_ARGUMENT],
    ['--decreasing', '-d', GetoptLong::NO_ARGUMENT],
    ['--quiet', '-q', GetoptLong::NO_ARGUMENT],
    ['--load', '-l', GetoptLong::REQUIRED_ARGUMENT],
    ['--time', '-t', GetoptLong::REQUIRED_ARGUMENT],
    ['--fail', '-f', GetoptLong::NO_ARGUMENT],
    ['--poll', '-p', GetoptLong::REQUIRED_ARGUMENT]
)

# Parsing the docs here
opts.each do |opt, arg|
    case opt
    when '--help'
        RDoc::usage(2)
    when '--time'
        maxTimeToWait = Time.now.to_i + arg.to_i
    when '--quiet'
        quiet = true
    when '--load'
        expected = arg.to_f
        optsSatisfied = true;
    when '--decreasing'
        loadDecreasing = true
        optsSatisfied = true;
    when '--poll'
        pollTime = arg.to_f
    when '--fail'
        failOnTimeout = true
    end
end

# and if we are not satisfied about parse results ..
if optsSatisfied == false then
    RDoc::usage(2,'Usage')
end

# and here we have the bread and butter
begin
    outBuf = ""
    open('/proc/loadavg') { |input|
        str = input.readline
        loadAvg = str.split(/ /)
        if loadDecreasing then
            if  loadAvg[0].to_f <= expected and loadAvg[1].to_f >= loadAvg[0].to_f then
                exitVal = 0
                looping = false
            else
                outBuf = loadAvg[0].to_s + " " + loadAvg[1].to_s
            end
        else
            if expected >= loadAvg[0].to_f  then
                exitVal = 0
                looping = false
            else 
                outBuf = loadAvg[0].to_s
            end
        end
    }

    if maxTimeToWait != 0 then
        current = Time.now.to_i
        if current >=maxTimeToWait then
            timeout = true
            looping = false
        else
            outBuf = outBuf + " " + (maxTimeToWait - current).to_s
        end
    end

    if not quiet then
        puts outBuf + "\n"
    end

    if looping then
        sleep pollTime
    end

end while looping

if timeout == true and failOnTimeout == true then
 exitVal = 1
end

exit(exitVal)
